#include "mafenetre.h"

MaFenetre::MaFenetre(QWidget *parent) :
    QWidget(parent)
{
    // La GUI
    labelListe = new QLabel("Interfaces :", this);
    listeInterfaces = new QComboBox(this);
    labelPeripherique = new QLabel("Périphérique :", this);
    editPeripherique = new QLineEdit(this);
    boutonAjouter = new QPushButton("Ajouter", this);
    boutonSupprimer = new QPushButton("Supprimer", this);

    QVBoxLayout *layoutPrincipal = new QVBoxLayout;
    QHBoxLayout *layoutListe = new QHBoxLayout;
    QHBoxLayout *layoutPeripherique = new QHBoxLayout;

    layoutListe->addStretch();
    layoutListe->addWidget(labelListe);
    layoutListe->addWidget(listeInterfaces);
    layoutListe->addWidget(boutonSupprimer);
    layoutPeripherique->addStretch();
    layoutPeripherique->addWidget(labelPeripherique);
    layoutPeripherique->addWidget(editPeripherique);
    layoutPeripherique->addWidget(boutonAjouter);

    layoutPrincipal->addLayout(layoutListe);
    layoutPrincipal->addLayout(layoutPeripherique);
    layoutPrincipal->addStretch();

    setLayout(layoutPrincipal);

    // Initialisation des données
    connect(boutonAjouter, SIGNAL(clicked()), this, SLOT(ajouter()));
    connect(boutonSupprimer, SIGNAL(clicked()), this, SLOT(supprimer()));
    if(lireXML())
    {
        afficherListe();
    }
}

MaFenetre::~MaFenetre()
{

}

bool MaFenetre::lireXML()
{
    // Le fichier XML
    QFile fichierXML(INTERFACES_XML);

    if(!(fichierXML.open(QIODevice::ReadOnly)))
    {
        QMessageBox::critical(0,"Erreur","Le fichier XML n'a pas pu être ouvert !");
        return false;
    }
    else
    {
        QDomDocument documentXML;

        documentXML.setContent(&fichierXML, false);
        QDomElement racine = documentXML.documentElement(); // <interfaces>
        if(racine.isNull())
        {
            qDebug() << "<MaFenetre::lireXML()> erreur racine !";
            return false;
        }

        QDomNode noeudInterface = racine.firstChild();
        if(noeudInterface.isNull())
        {
            qDebug() << "<MaFenetre::lireXML()> erreur racine vide !";
            return false;
        }

        while(!noeudInterface.isNull())
        {
            Interface interface;
            QDomElement elementInterface = noeudInterface.toElement(); // <interface>
            if(elementInterface.isNull())
            {
                qDebug() << "<MaFenetre::lireXML()> erreur element interface !";
                break;
            }

            QDomNode noeudPeripherique = elementInterface.firstChild();
            if(!noeudPeripherique.isNull())
            {
                QDomElement elementPeripherique = noeudPeripherique.toElement(); // <peripherique>

                interface.id = elementInterface.attribute("id").toInt(); // l'id
                interface.peripherique = elementPeripherique.text(); // le device
                qDebug() << QString::fromUtf8("<MaFenetre::lireXML()> %1 -> %2").arg(QString::number(interface.id)).arg(interface.peripherique);
                interfaces.push_back(interface);
            }
            else qDebug() << "<MaFenetre::lireXML()> erreur noeud peripherique !";

            noeudInterface = noeudInterface.nextSibling();
        }

        fichierXML.close();
    }

    return true;
}

bool MaFenetre::ecrireXML()
{
    // Le fichier XML
    QFile fichierXML(INTERFACES_XML);
    //QFile fichierXML("test.xml");

    if(!(fichierXML.open(QIODevice::ReadWrite | QIODevice::Truncate)))
    {
        QMessageBox::critical(0,"Erreur","Erreur ouverture fichier XML !");
        return false;
    }
    else
    {
        QDomDocument documentXML;

        documentXML.setContent(&fichierXML, false);
        QDomNode xmlNode = documentXML.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
        documentXML.insertBefore(xmlNode, documentXML.firstChild());

        if(interfaces.size() > 0)
        {
            QDomElement root = documentXML.createElement("interfaces");
            documentXML.appendChild(root);

            for(int i = 0; i < interfaces.size(); i++)
            {
                QDomElement elementInterface = documentXML.createElement("interface");
                elementInterface.setAttribute("id", interfaces[i].id);
                root.appendChild(elementInterface);

                QDomElement elementPeripherique = documentXML.createElement("peripherique");
                elementInterface.appendChild(elementPeripherique);

                QDomText text = documentXML.createTextNode(interfaces[i].peripherique);
                elementPeripherique.appendChild(text);
            }
        }

        QTextStream out(&fichierXML);
        documentXML.save(out, 2);

        fichierXML.close();
    }

    return true;
}

void MaFenetre::ajouter()
{
    if(!editPeripherique->text().isEmpty())
    {
        Interface interface;
        interface.id = listeInterfaces->count();
        interface.peripherique = editPeripherique->text();
        qDebug() << QString::fromUtf8("<MaFenetre::ajouter()> %1 -> %2").arg(QString::number(interface.id)).arg(interface.peripherique);
        interfaces.push_back(interface);
        listeInterfaces->insertItem(interface.id, interface.peripherique);
        ecrireXML();
    }
}

void MaFenetre::supprimer()
{
    if(listeInterfaces->currentIndex() >= 0)
    {
        qDebug() << "<MaFenetre::supprimer()> suppression " << interfaces.at(listeInterfaces->currentIndex()).peripherique;
        interfaces.removeAt(listeInterfaces->currentIndex());
        listeInterfaces->removeItem(listeInterfaces->currentIndex());
        ecrireXML();
    }
}

void MaFenetre::afficherListe()
{
    for(int i = 0; i < interfaces.size(); i++)
    {
        listeInterfaces->insertItem(interfaces[i].id, interfaces[i].peripherique);
    }
}
